@@ -229,39 +229,70 @@ sub addCommandLineOption ($) {
 
 sub removeHaplotypeOptions () {
     my @strippedOpts;
-    my $setUpForPacBio    = 0;
-    my $setUpForNanopore  = 0;
+
     my $haveRaw           = 0;
     my $haveCorrected     = 0;
 
+    my $haveTrimmed       = 0;
+
+    my $setUpForPacBio    = 0;
+    my $setUpForNanopore  = 0;
+    my $setUpForHiFi      = 0;
+
     #  A very specialized function.  Remove all the sequence file options,
     #  both long reads and short reads used for haplotyping, from the list of
-    #  command line options.  Then return a string appropriate for adding new
-    #  long reads.
+    #  command line options.  Then return a string appropriate for assembling
+    #  the haplotyped reads.
     #
     #  Note that when this is called a second (third, etc) time, the
     #  semantics are different (no haplotype options to remove) but the end
     #  result is the same.
 
     foreach my $opt (@cLineOpts) {
-        if ($opt =~ m/^-pacbio-raw\s/)          { $setUpForPacBio++;   $haveRaw++;       next; }
-        if ($opt =~ m/^-pacbio-corrected\s/)    { $setUpForPacBio++;   $haveCorrected++; next; }
-        if ($opt =~ m/^-nanopore-raw\s/)        { $setUpForNanopore++; $haveRaw++;       next; }
-        if ($opt =~ m/^-nanopore-corrected\s/)  { $setUpForNanopore++; $haveCorrected++; next; }
-        if ($opt =~ m/^-pacbio-hifi\s/)         { $setUpForPacBio++;   $haveCorrected++; next; }
-        if ($opt =~ m/^-haplotype/)             {                                        next; }
-        if ($opt =~ m/^-d\s/)                   {                                        next; }
-        if ($opt =~ m/^-p\s/)                   {                                        next; }
+        # for v1.*
+        if ($opt =~ m/^-pacbio-raw\s/)          { $haveRaw++;          $setUpForPacBio++;     next; }
+        if ($opt =~ m/^-pacbio-corrected\s/)    { $haveCorrected++;    $setUpForPacBio++;     next; }
+        if ($opt =~ m/^-nanopore-raw\s/)        { $haveRaw++;          $setUpForNanopore++;   next; }
+        if ($opt =~ m/^-nanopore-corrected\s/)  { $haveCorrected++;    $setUpForNanopore++;   next; }
+
+        if ($opt =~ m/^-raw\s/)                 { $haveRaw++;          next; }
+        if ($opt =~ m/^-corrected\s/)           { $haveCorrected++;    next; }
+
+        if ($opt =~ m/^-trimmed\s/)             { $haveTrimmed++;      next; }
+
+        if ($opt =~ m/^-pacbio\s/)              { $setUpForPacBio++;   next; }
+        if ($opt =~ m/^-nanopore\s/)            { $setUpForNanopore++; next; }
+        if ($opt =~ m/^-pacbio-hifi\s/)         { $setUpForHiFi++;     next; }
+
+        if ($opt =~ m/^-haplotype/)             {                      next; }
+        if ($opt =~ m/^-d\s/)                   {                      next; }
+        if ($opt =~ m/^-p\s/)                   {                      next; }
 
         push @strippedOpts, $opt;
     }
 
-    my $tech = ($setUpForNanopore > 0) ? "-nanopore"  : "-pacbio";
-    my $type = ($haveCorrected    > 0) ? "-corrected" : "-raw";
-
     @cLineOpts = @strippedOpts;
 
-    return("$tech$type");
+    #  Now figure out what to load the haplotyped reads as.
+
+    my $tech = "";
+
+    #  Either raw or corrected (or hifi).
+
+    if    ($haveRaw)           {  $tech .= " ";             }    # implicit
+    elsif ($haveCorrected)     {  $tech .= "-corrected ";   }
+
+    #  And they can be trimmed or not.
+
+    if    ($haveTrimmed)       {  $tech .= "-trimmed ";     }
+
+    #  Only one tech is allowed.
+
+    if    ($setUpForHiFi)      {  $tech .= "-pacbio-hifi";  }
+    elsif ($setUpForNanopore)  {  $tech .= "-nanopore";     }
+    elsif ($setUpForPacBio)    {  $tech .= "-pacbio";       }
+
+    return($tech);
 }
 
 
@@ -401,10 +432,10 @@ sub printHelp (@) {
     print "               genomeSize=<number>[g|m|k] \\\n";
     print "              [other-options] \\\n";
     print "              [-haplotype{NAME} illumina.fastq.gz] \\\n";
-    print "              [-pacbio-raw |\n";
-    print "               -pacbio-corrected |\n";
-    print "               -nanopore-raw |\n";
-    print "               -nanopore-corrected |\n";
+    print "              [-corrected] \\\n";
+    print "              [-trimmed] \\\n";
+    print "              [-pacbio |\n";
+    print "               -nanopore |\n";
     print "               -pacbio-hifi] file1 file2 ...\n";
     print "\n";
     print "example: canu -d run1 -p godzilla genomeSize=1g -nanopore-raw reads/*.fasta.gz \n";
@@ -455,12 +486,17 @@ sub printHelp (@) {
     print "    -haplotypeBILLY billy1.fasta.gz billy2.fasta.gz\n";
     print "\n";
     print "  Reads can be either FASTA or FASTQ format, uncompressed, or compressed with gz, bz2 or xz.\n";
-    print "  Reads are specified by the technology they were generated with, and any processing performed:\n";
-    print "    -pacbio-raw         <files>      Reads are straight off the machine.\n";
-    print "    -pacbio-corrected   <files>      Reads have been corrected.\n";
-    print "    -nanopore-raw       <files>\n";
-    print "    -nanopore-corrected <files>\n";
-    print "    -pacbio-hifi        <files>\n";
+    print "\n";
+    print "  Reads are specified by the technology they were generated with, and any processing performed.\n";
+    print "\n";
+    print "  [processing]\n";
+    print "    -corrected\n";
+    print "    -trimmed\n";
+    print "\n";
+    print "  [technology]\n";
+    print "    -pacbio      <files>\n";
+    print "    -nanopore    <files>\n";
+    print "    -pacbio-hifi <files>\n";
     print "\n";
     print "Complete documentation at http://canu.readthedocs.org/en/latest/\n";
     print "\n";
@@ -468,9 +504,10 @@ sub printHelp (@) {
     if (defined($global{'errors'})) {
         print "$global{'errors'}";
         print "\n";
-    }
-
-    exit(1);
+        exit(1);
+    } else {
+       exit(0);
+   }
 }
 
 
@@ -515,13 +552,6 @@ sub printCitation ($) {
        print STDERR "${prefix}  https://arxiv.org/abs/1708.01492\n";
        print STDERR "${prefix}\n";
     }
-    if (getGlobal("unitigger") eq "wtdbg") {
-       print STDERR "${prefix}Contigs are constructed use:\n";
-       print STDERR "${prefix}  Ruan J.\n";
-       print STDERR "${prefix}  WTDBG A fuzzy Bruijn graph (FBG) approach to long noisy reads assembly.\n";
-       print STDERR "${prefix}  https://github.com/ruanjue/wtdbg\n";
-       print STDERR "${prefix}\n";
-    }
 
     print STDERR "${prefix}Corrected read consensus sequences are generated using an algorithm derived from FALCON-sense:\n";
     print STDERR "${prefix}  Chin CS, et al.\n";
@@ -647,6 +677,8 @@ sub setParametersFromFile ($$@) {
         my $two;
         my $opt;
 
+        #  COMPATIBILITY MODE!
+
         if (m/^-(pacbio|nanopore)-(corrected|raw)\s+(.*)\s*$/) {   #  Comments not allowed, because then we can't decide
             $one  = "-$1-$2";                                      #  if the # is a comment, or part of the file!
             $two = $3;                                             #  e.g.,   this_is_file_#1   vs
@@ -878,7 +910,8 @@ sub setDefaults () {
     setDefault("minOverlapLength",     500,       "Overlaps shorter than this length are not computed; default 500");
 
     setDefault("readSamplingBias",     0.0,       "Score reads as 'random * length^bias', keep the highest scoring reads");
-    setDefault("readSamplingCoverage", undef,     "Discard reads to make the input be of this size");
+    setDefault("readSamplingCoverage", undef,     "DEPRECATED; use maxInputCoverage.  Discard reads to make the input be of this size");
+    #  DEPRECATED - use minInputCoverage and maxInputCoverage instead
 
     setDefault("minMemory",            undef,     "Minimum amount of memory needed to compute the assembly (do not set unless prompted!)");
     setDefault("maxMemory",            undef,     "Maximum memory to use by any component of the assembler");
@@ -886,11 +919,13 @@ sub setDefaults () {
     setDefault("minThreads",           undef,     "Minimum number of compute threads suggested to compute the assembly");
     setDefault("maxThreads",           undef,     "Maximum number of compute threads to use by any component of the assembler");
 
+    setDefault("minInputCoverage",     10,        "Stop if input coverage is too low; default 10");
+    setDefault("maxInputCoverage",     undef,     "If input coverage is high, downsample to something reasonable; default 200");
+
     #####  Stopping conditions
 
-    setDefault("stopOnReadQuality", 1,     "Stop if a significant portion of the input data has quality value or base composition errors");
-    setDefault("stopOnLowCoverage", 10,    "Stop if raw, corrected or trimmed read coverage is low");
-    setDefault("stopAfter",         undef, "Stop after a specific algorithm step is completed");
+    setDefault("stopOnLowCoverage",    10,        "Stop if raw, corrected or trimmed read coverage is low");
+    setDefault("stopAfter",            undef,     "Stop after a specific algorithm step is completed");
 
     #####  Grid Engine configuration, internal parameters.  These are filled out in canu.pl, right after this function returns.
 
@@ -901,6 +936,8 @@ sub setDefaults () {
     setDefault("gridEngineArrayName",                 undef, "Grid engine configuration, not documented");
     setDefault("gridEngineArrayMaxJobs",              undef, "Grid engine configuration, not documented");
     setDefault("gridEngineOutputOption",              undef, "Grid engine configuration, not documented");
+    setDefault("gridEngineThreadsOption",             undef, "Grid engine configuration, not documented");
+    setDefault("gridEngineMemoryOption",              undef, "Grid engine configuration, not documented");
     setDefault("gridEngineResourceOption",            undef, "Grid engine configuration, not documented");
     setDefault("gridEngineMemoryUnits",               undef, "Grid engine configuration, not documented");
     setDefault("gridEngineMemoryPerJob",              undef, "Grid engine configuration, not documented");
@@ -1009,24 +1046,18 @@ sub setDefaults () {
 
     #####  Unitigger & BOG & bogart Options
 
-    setDefault("unitigger",      "bogart", "Which unitig algorithm to use; 'bogart' or 'wtdbg' supported; default 'bogart'");
+    setDefault("unitigger",      "bogart", "Which unitig algorithm to use; only 'bogart' supported; default 'bogart'");
     setDefault("genomeSize",     undef, "An estimate of the size of the genome");
     setDefault("batOptions",     undef, "Advanced options to bogart");
     setDefault("batMemory",      undef, "Approximate maximum memory usage, in gigabytes, default is the maxMemory limit");
     setDefault("batThreads",     undef, "Number of threads to use; default is the maxThreads limit");
     setDefault("batConcurrency", undef, "Unused, only one process supported");
-    setDefault("dbgOptions",     undef, "Advanced options to wtdbg");
-    setDefault("dbgMemory",      undef, "Approximate maximum memory usage, in gigabytes, default is the maxMemory limit");
-    setDefault("dbgThreads",     undef, "Number of threads to use; default is the maxThreads limit");
-    setDefault("dbgConcurrency", undef, "Unused, only one process supported");
 
     setDefault("contigFilter",   "2 0 1.0 0.5 3",   "Parameters to filter out 'unassembled' unitigs.  Five values: minReads minLength singleReadSpan lowCovFraction lowCovDepth");
 
     #####  Consensus Options
 
-    setDefault("cnsPartitions",   undef,       "Partition consensus into N jobs");
-    setDefault("cnsPartitionMin", undef,       "Don't make a consensus partition with fewer than N reads");
-    setDefault("cnsMaxCoverage",  40,          "Limit unitig consensus to at most this coverage; default '0' = unlimited");
+    setDefault("cnsMaxCoverage",  40,          "Limit unitig consensus to at most this coverage; default '40' = unlimited");
     setDefault("cnsConsensus",    "pbdagcon",  "Which consensus algorithm to use; 'pbdagcon' (fast, reliable); 'utgcns' (multialignment output); 'quick' (single read mosaic); default 'pbdagcon'");
 
     #####  Correction Options
@@ -1042,6 +1073,8 @@ sub setDefaults () {
     setDefault("corFilter",                    "expensive",  "Method to filter short reads from correction; 'quick' or 'expensive'; default 'expensive'");
     setDefault("corConsensus",                 "falcon",     "Which consensus algorithm to use; only 'falcon' is supported; default 'falcon'");
 
+    setDefault("homoPolyCompress",             undef,        "Compute everything but consensus sequences using homopolymer compressed reads");
+
     #  Convert all the keys to lowercase, and remember the case-sensitive version
 
     foreach my $k (keys %synops) {
@@ -1502,9 +1535,8 @@ sub checkParameters () {
         }
     }
 
-    if ((getGlobal("unitigger") ne "bogart") &&
-        (getGlobal("unitigger") ne "wtdbg")) {
-        addCommandLineError("ERROR:  Invalid 'unitigger' specified (" . getGlobal("unitigger") . "); must be 'wtdbg' or 'bogart'\n");
+    if (getGlobal("unitigger") ne "bogart") {
+        addCommandLineError("ERROR:  Invalid 'unitigger' specified (" . getGlobal("unitigger") . "); must be 'bogart'\n");
     }
 
     if ((getGlobal("corConsensus") ne "falcon")) {
@@ -1517,10 +1549,32 @@ sub checkParameters () {
         addCommandLineError("ERROR:  Invalid 'cnsConsensus' specified (" . getGlobal("cnsConsensus") . "); must be 'quick', 'pbdagcon', or 'utgcns'\n");
     }
 
+    if (defined(getGlobal("readSamplingCoverage"))) {
+        print STDERR "--\n";
+        print STDERR "--  WARNING:  Deprecated option 'readSamplingCoverage' supplied.\n";
+        print STDERR "--  WARNING:  Use 'maxInputCoverage' instead.\n";
+        print STDERR "--  WARNING:  'readSamplingCoverage' will be removed in the next release.\n";
+        print STDERR "--\n";
 
-    if ((!defined("lowCoverageAllowed") &&  defined("lowCoverageDepth")) ||
-        ( defined("lowCoverageAllowed") && !defined("lowCoverageDepth"))) {
-        addCommandLineError("ERROR:  Invalid 'lowCoverageAllowed' and 'lowCoverageDepth' specified; both must be set\n");
+        setGlobal("maxInputCoverage", getGlobal("readSamplingCoverage"));
+    }
+    if (!defined(getGlobal("maxInputCoverage"))) {
+       setGlobal("maxInputCoverage", 200);
+    }
+    if (getGlobal("maxInputCoverage") eq "all") {
+        setGlobal("maxInputCoverage", 0);
+    }
+    if ((getGlobal("maxInputCoverage") > 0) && (getGlobal("maxInputCoverage") < getGlobal("minInputCoverage"))) {
+        my $minc = getGlobal("minInputCoverage");
+        my $maxc = getGlobal("maxInputCoverage");
+
+        addCommandLineError("ERROR:  minInputCoverage ($minc) must be less than maxInputCoverage ($maxc).\n");
+    }
+    if ((getGlobal("maxInputCoverage") > 0) && (getGlobal("maxInputCoverage") < getGlobal("stopOnLowCoverage"))) {
+        my $minc = getGlobal("stopOnLowCoverage");
+        my $maxc = getGlobal("maxInputCoverage");
+
+        addCommandLineError("ERROR:  stopOnLowCoverage ($minc) must be less than maxInputCoverage ($maxc).\n");
     }
 
     if ((getGlobal("saveOverlaps") ne "0") &&
@@ -1563,7 +1617,7 @@ sub checkParameters () {
         my $st = getGlobal("stopAfter");
         $st =~ tr/A-Z/a-z/;
 
-        $st = "correctiom"   if ($st eq "readcorrection");    #  Update the string to allow deprecated usage.
+        $st = "correction"   if ($st eq "readcorrection");    #  Update the string to allow deprecated usage.
         $st = "trimming"     if ($st eq "readtrimming");
 
         my $failureString = "ERROR:  Invalid stopAfter specified (" . getGlobal("stopAfter") . "); must be one of:\n";
