@@ -46,7 +46,7 @@ require Exporter;
              fetchFileFromLink    fetchFileFromLinkShellCode
              stashFile            stashFileShellCode
                                   stashFilesShellCode
-             fetchSeqStore        fetchSeqStoreShellCode   fetchSeqStorePartitionShellCode
+             fetchSeqStore        fetchSeqStoreShellCode
              fetchOvlStore        fetchOvlStoreShellCode
              fetchTigStore        fetchTigStoreShellCode
              stashSeqStore
@@ -67,7 +67,7 @@ use canu::Execution;
 
 #use canu::Grid "formatAllowedResources";
 
-my $showWork   = 1;
+my $showWork = 1;
 
 my $retryCount = 5;    #  Try a few times to upload the file.
 my $retryDelay = 10;   #  Wait a little bit before retrying.
@@ -640,16 +640,19 @@ sub stashSeqStore ($) {
         #  we won't upload the metadata, and the store will need to be regenerated.
 
         $files  = "./$asm.seqStore.err ";
-        $files .= "./$asm.seqStore.ssi ";
+        $files .= "./$asm.seqStore.sh ";
         $files .= "./$asm.seqStore/errorLog ";
         $files .= "./$asm.seqStore/info ";
         $files .= "./$asm.seqStore/info.txt ";
         $files .= "./$asm.seqStore/libraries ";
         $files .= "./$asm.seqStore/libraries.txt ";
-        $files .= "./$asm.seqStore/load.dat ";
         $files .= "./$asm.seqStore/readNames.txt ";
         $files .= "./$asm.seqStore/readlengths* ";
         $files .= "./$asm.seqStore/reads ";
+        $files .= "./$asm.seqStore/reads-corc ";
+        $files .= "./$asm.seqStore/reads-coru ";
+        $files .= "./$asm.seqStore/reads-rawc ";
+        $files .= "./$asm.seqStore/reads-rawu ";
         $files .= "./$asm.seqStore/version*"      if (-e "./$asm.seqStore/version.001");
 
         if (runCommandSilently(".", "tar -cf - $files | gzip -1c > ./$asm.seqStore.tar.gz", 1)) {
@@ -754,35 +757,6 @@ sub stashSeqStorePartitions ($$$$) {
 }
 
 
-sub fetchSeqStorePartitionShellCode ($$$) {
-    my $asm    = shift @_;           #  The name of the assembly.
-    my $path   = shift @_;           #  The subdir we're running in; 'unitigging/4-unitigger', etc.
-    my $indent = shift @_;           #
-    my $base   = dirname($path);     #  'unitigging'
-    my $root   = pathToDots($path);  #  '../..'
-    my $code   = "";
-
-    my $storePath = "$base/$asm.\${tag}Store";
-    my $storeName = "partitionedReads.seqStore";
-
-    if (defined(isOS())) {
-        $code .= "\n";
-        $code .= "${indent}if [ ! -e $root/$storePath/$storeName/partitions/blobs.\$jobid ] ; then\n";
-        $code .= "${indent}  cd $root/$storePath\n";
-        $code .= fetchFileShellCode($storePath, "$storeName.\$jobid.tar.gz", "${indent}  ");
-        $code .= "${indent}  gzip -dc ./$storeName.\$jobid.tar.gz | tar -xf -\n";
-        $code .= "${indent}  rm -f ./$storeName.\$jobid.tar.gz\n";
-        $code .= "${indent}  cd -\n";
-        $code .= "${indent}fi\n";
-    }
-
-    return($code);
-}
-
-
-
-
-
 
 sub stashOvlStore ($$) {
     my $asm    = shift @_;
