@@ -75,7 +75,7 @@ sub haplotypeReadsExist ($@) {
     #  If no haplotypes, no haplotype reads exist.  Of note, this string causes the canu
     #  main to skip all haplotype processing.
 
-    if (scalar(@haplotypes) == 0){
+    if (scalar(@haplotypes) == 0) {
         return("no-haplotypes");
     }
 
@@ -85,6 +85,8 @@ sub haplotypeReadsExist ($@) {
         return("no")   if (! fileExists("./haplotype/haplotype-$haplotype.fasta.gz"));
     }
 
+    return("no")       if (! fileExists("./haplotype/haplotyping.success"));
+
     #  Yeah!  Reads exist!
 
     return("yes");
@@ -196,6 +198,7 @@ sub haplotypeSplitReads ($$%) {
         }
 
         close(OUT);
+
         stashFile("$path/reads-$haplotype/reads-$haplotype-$fileNumber.fasta.gz");
 
         open(OUT, "> $path/reads-$haplotype/reads-$haplotype.success");
@@ -452,7 +455,7 @@ sub haplotypeCountConfigure ($%) {
     print F "\n";
     print F "#  And compute.\n";
     print F "\n";
-    print F "$bin/meryl k=$merSize threads=$thr memory=$mem \\\n";
+    print F "\$bin/meryl k=$merSize threads=$thr memory=$mem \\\n";
     print F "  count \\\n";
     print F "    output ./reads-\$output.meryl.WORKING \\\n";
     print F "    \$batch \\\n";
@@ -531,7 +534,7 @@ sub haplotypeCountConfigure ($%) {
     print F "\n";
     print F "#  And compute.\n";
     print F "\n";
-    print F "$bin/meryl threads=$thr memory=$mem \\\n";
+    print F "\$bin/meryl threads=$thr memory=$mem \\\n";
     print F "  union-sum \\\n";
     print F "    output ./reads-\$haplotype.meryl.WORKING \\\n";
     print F "    ./reads-\$haplotype-???.meryl \\\n";
@@ -540,7 +543,7 @@ sub haplotypeCountConfigure ($%) {
     print F "&& \\\n";
     print F "rm -rf ./reads-\$haplotype-???.meryl\n";
     print F "\n";
-    print F "$bin/meryl threads=$thr memory=$mem \\\n";
+    print F "\$bin/meryl threads=$thr memory=$mem \\\n";
     print F "  statistics \\\n";
     print F "    ./reads-\$haplotype.meryl \\\n";
     print F "> ./reads-\$haplotype.statistics \\\n";
@@ -625,7 +628,7 @@ sub haplotypeCountConfigure ($%) {
     print F "\n";
     print F "#  Subtract all the other haplotypes from ours.\n";
     print F "\n";
-    print F "$bin/meryl threads=$thr memory=$mem \\\n";
+    print F "\$bin/meryl threads=$thr memory=$mem \\\n";
     print F "  difference \\\n";
     print F "    output ./haplotype-\$haplotype.meryl.WORKING \\\n";
     print F "    ./reads-\$haplotype.meryl \\\n";
@@ -1049,7 +1052,7 @@ sub haplotypeReadsConfigure ($@) {
     print F "\n";
     print F "#  Assign reads to haplotypes.\n";
     print F "\n";
-    print F "$bin/splitHaplotype \\\n";
+    print F "\$bin/splitHaplotype \\\n";
     print F "  -cl $minReadLength \\\n";
     print F "  -threads $thr \\\n";
     print F "  -memory  $mem \\\n";
@@ -1058,7 +1061,7 @@ sub haplotypeReadsConfigure ($@) {
     print F "  -A ./haplotype-unknown.fasta.gz \\\n";
     print F "> haplotype.log.WORKING \\\n";
     print F "&& \\\n";
-    print F "mv -f ./haplotype.log.WORKING ./haplotype.log \\\n";
+    print F "mv -f ./haplotype.log.WORKING ./haplotype.log\n";
 
     if (defined(getGlobal("objectStore"))) {
         print F "\n";
@@ -1085,6 +1088,11 @@ sub haplotypeReadsConfigure ($@) {
 
 
 
+#  Checks for presence of haplotype.log, and if that exists,
+#  declares the haplotyping stage done and creates haplotping.success.
+#
+#  If haplotyping failed, the (partial) log will be in haplotype.log.WORKING.
+#
 sub haplotypeReadsCheck ($@) {
     my $asm        = shift @_;
     my @haplotypes =       @_;
@@ -1140,7 +1148,7 @@ sub haplotypeReadsCheck ($@) {
             print STDERR "-- Haplotyping jobs failed, tried $attempt times, giving up.\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
+            caExit(undef, "$path/haplotype.log.WORKING");
         }
 
         if ($attempt > 0) {
