@@ -69,24 +69,36 @@ sub loadReadLengths ($$$) {
 
     my $bin     = getBinDirectory();
 
-    $$rlVec     = "\xff" x ($maxID * 4 + 4);
+    $$rlVec     = "\x00" x ($maxID * 4 + 4);
 
     print STDERR "--\n";
     print STDERR "-- Loading read lengths.\n";
 
-    open(F, "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -reads 2> /dev/null |");
+    #  Dump corrected read lengths and trimming.  Load the length into
+    #  an array for later use.
+
+    open(F, "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -corrected -reads 2> /dev/null |");
     while (<F>) {
         s/^\s+//;
         s/\s+$//;
 
         next if (m/^readID/);   #  Header line
-        next if (m/^------/);   #  ------ ----
+        next if (m/^----/);     #  ------ ----
 
         my @v = split '\s+', $_;
+        my $l = 0;
 
-        vec($$rlVec, $v[0], 32) = $v[2];
+        if ($v[5] =~ m/CC--/) {   #  Corrected available, but no trimmed.
+            $l = $v[2];
+        }
+
+        if ($v[5] =~ m/CCTT/) {   #  Corrected and trimmed.
+            $l = $v[4] - $v[3];
+        }
+
+        vec($$rlVec, $v[0], 32) = $l;
         $rlCnt                 += 1;
-        $rlSum                 += $v[2];
+        $rlSum                 += $l;
     }
     close(F);
 
@@ -596,6 +608,7 @@ sub overlapErrorAdjustmentConfigure ($) {
     print F "  -O ../$asm.ovlStore \\\n";
     print F "  -R \$minid \$maxid \\\n";
     print F "  -e " . getGlobal("utgOvlErrorRate") . " -l " . getGlobal("minOverlapLength") . " \\\n";
+    print F "  -s \\\n"                                   if (defined(getGlobal("homoPolyCompress")));
     print F "  -c ./red.red \\\n";
     print F "  -o ./\$jobid.oea.WORKING \\\n";
     print F "&& \\\n";
