@@ -30,6 +30,7 @@ require Exporter;
 @ISA    = qw(Exporter);
 @EXPORT = qw(getNumberOfReadsInStore
              getNumberOfBasesInStore
+             getSequenceStoreStats
              getSizeOfSequenceStore
              getExpectedCoverage
              generateReadLengthHistogram
@@ -73,13 +74,18 @@ sub getNumberOfReadsInStore ($$) {
 
     open(F, "< ./$asm.seqStore/info.txt") or caExit("can't open './$asm.seqStore/info.txt' for reading: $!", undef);
     while (<F>) {
-        $nr = $1    if ((m/numReads\s+=\s+(\d+)/)          && ($tag eq "all"));
-        $nr = $1    if ((m/numRawReads\s+=\s+(\d+)/)       && ($tag eq "cor" || $tag eq "hap"));
-        $nr = $1    if ((m/numCorrectedReads\s+=\s+(\d+)/) && ($tag eq "obt"));
-        $nr = $1    if ((m/numTrimmedReads\s+=\s+(\d+)/)   && ($tag eq "utg"));
+        if (m/^\s*(\d+)\s+([0123456789-]+)\s+(.*)\s*$/) {
+            $nr = $1 if (($3 eq "total-reads")       && ($tag eq "all"));
+            $nr = $1 if (($3 eq "raw")               && ($tag eq "cor"));
+            $nr = $1 if (($3 eq "raw")               && ($tag eq "hap"));
+            $nr = $1 if (($3 eq "corrected")         && ($tag eq "obt"));
+            $nr = $1 if (($3 eq "corrected-trimmed") && ($tag eq "utg"));
+        }
     }
     close(F);
 
+    #print STDERR "-- Found $nr reads in '$asm.seqStore', for tag '$tag'\n";
+
     return($nr);
 }
 
@@ -100,9 +106,12 @@ sub getNumberOfBasesInStore ($$) {
 
     open(F, "< ./$asm.seqStore/info.txt") or caExit("can't open './$asm.seqStore/info.txt' for reading: $!", undef);
     while (<F>) {
-        $nb = $1    if ((m/numRawBases\s+=\s+(\d+)/)       && ($tag eq "cor" || $tag eq "hap"));
-        $nb = $1    if ((m/numCorrectedBases\s+=\s+(\d+)/) && ($tag eq "obt"));
-        $nb = $1    if ((m/numTrimmedBases\s+=\s+(\d+)/)   && ($tag eq "utg"));
+        if (m/^\s*(\d+)\s+(\d+)\s+(.*)\s*$/) {
+            $nb = $2 if (($3 eq "raw")               && ($tag eq "cor"));
+            $nb = $2 if (($3 eq "raw")               && ($tag eq "hap"));
+            $nb = $2 if (($3 eq "corrected")         && ($tag eq "obt"));
+            $nb = $2 if (($3 eq "corrected-trimmed") && ($tag eq "utg"));
+        }
     }
     close(F);
 
@@ -111,6 +120,65 @@ sub getNumberOfBasesInStore ($$) {
 
 
 
+sub getSequenceStoreStats ($) {
+    my $asm    = shift @_;
+    my $bin    = getBinDirectory();
+
+    my ($numRaw, $numRawTri, $numCor, $numCorTri) = (0, 0, 0, 0);
+    my ($numPacBio, $numNanopore, $numHiFi)       = (0, 0, 0);
+
+    #  Recreate our metadata dumps, if needed.
+
+    if (! -d "./$asm.seqStore") {
+        return(0, 0, 0, 0, 0, 0, 0);
+    }
+
+    if (! -e "./$asm.seqStore/info.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -stats > ./$asm.seqStore/info.txt 2> /dev/null", 1)) {
+            caExit("failed to generate $asm.seqStore/info.txt", undef);
+        }
+    }
+
+    if (! -e "./$asm.seqStore/libraries.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -libs > ./$asm.seqStore/libraries.txt 2> /dev/null", 1)) {
+            caExit("failed to generate $asm.seqStore/libraries.txt", undef);
+        }
+    }
+
+    #  Count the number of reads or each type.
+
+    open(L, "< ./$asm.seqStore/info.txt") or caExit("can't open './$asm.seqStore/info.txt' for reading: $!", undef);
+    while (<L>) {
+        s/^\s+//;
+        s/\s+$//;
+
+        my @v = split '\s+', $_;
+
+        $numRaw++         if (($v[2] eq "raw")               && ($v[1] > 0));
+        $numRawTri++      if (($v[2] eq "raw-trimmed")       && ($v[1] > 0));
+        $numCor++         if (($v[2] eq "corrected")         && ($v[1] > 0));
+        $numCorTri++      if (($v[2] eq "corrected-trimmed") && ($v[1] > 0));
+    }
+    close(L);
+
+    open(L, "< ./$asm.seqStore/libraries.txt") or caExit("can't open './$asm.seqStore/libraries.txt' for reading: $!", undef);
+    while (<L>) {
+        s/^\s+//;
+        s/\s+$//;
+
+        my @v = split '\s+', $_;
+
+        $numPacBio++       if ($v[1] eq "PacBio");
+        $numNanopore++     if ($v[1] eq "Nanopore");
+        $numHiFi++         if ($v[1] eq "PacBioHiFi");
+    }
+    close(L);
+
+    return($numRaw, $numRawTri, $numCor, $numCorTri, $numPacBio, $numNanopore, $numHiFi);
+}
+
+
+
 sub getSizeOfSequenceStore ($) {
     my $asm    = shift @_;
     my $size   = 0;
@@ -145,71 +213,54 @@ sub createSequenceStore ($$@) {
     my $bin    = getBinDirectory();
     my @inputs = @_;
 
-    #  If the store failed to build because of input errors and warnings, rename the store and continue.
-    #  Not sure how to support this in DNANexus.
-
-    if (-e "./$asm.seqStore.ACCEPTED") {
-        rename("./$asm.seqStore.ACCEPTED",          "./$asm.seqStore");
-        rename("./$asm.seqStore.BUILDING.err",      "./$asm.seqStore.err");
-        return;
-    }
-
-    #  If the store failed to build and the user just reruns canu, this will be triggered.  We'll
-    #  skip rebuilding the store again, and report the original error message.
-
-    if (-e "./$asm.seqStore.BUILDING") {
-        print STDERR "-- WARNING:\n";
-        print STDERR "-- WARNING:  Previously failed seqStore detected.\n";
-        print STDERR "-- WARNING:\n";
-    }
-
-    #  Not sure how this can occur.  Possibly the user just deleted seqStore.BUILDING and restarted?
-
-    if ((! -e "./$asm.seqStore.BUILDING") && (-e "./$asm.seqStore.ssi")) {
-        print STDERR "-- WARNING:\n";
-        print STDERR "-- WARNING:  Existing sequence inputs used.\n";
-        print STDERR "-- WARNING:\n";
-    }
-
     #  Fail if there are no inputs.
 
     caExit("no input files specified, and store not already created, I have nothing to work on!", undef)
         if (scalar(@inputs) == 0);
 
-    #  Convert the canu-supplied reads into correct relative paths.  This is made complicated by
-    #  sqStoreCreate being run in a directory one below where we are now.
+    #  If any of the files are links to objects, fetch the object
+    #  to local disk and update the name.
+    #
+    #  A similar blcok is used in SequenceStore.pm and HaplotypeReads.pm (twice).
 
-    #  At the same time, check that all files exist.
+    for (my $ff=0; $ff < scalar(@inputs); $ff++) {
+        my ($inType, $inPath) = split '\0', $inputs[$ff];
 
-    if (! -e "./$asm.seqStore.ssi") {
+        if ($inPath =~ m/dnanexus:(.*)=(.*)/) {
+            my $link = $1;
+            my $name = $2;
 
-        #  If any of the files are links to objects, fetch the object
-        #  to local disk and update the name.
-        #
-        #  A similar blcok is used in SequenceStore.pm and HaplotypeReads.pm (twice).
+            print STDERR "-- Fetch input file './$name' from object '$link'\n";
 
-        for (my $ff=0; $ff < scalar(@inputs); $ff++) {
-            my ($inType, $inPath) = split '\0', $inputs[$ff];
+            fetchFileFromLink($link, $name);
 
-            if ($inPath =~ m/dnanexus:(.*)=(.*)/) {
-                my $link = $1;
-                my $name = $2;
+            $inputs[$ff] = "$inType\0./$name";
+        }
+    }
 
-                print STDERR "-- Fetch input file './$name' from object '$link'\n";
+    if (! -e "./$asm.seqStore.sh") {
+        open(F, "> ./$asm.seqStore.sh") or caExit("cant' open './$asm.seqStore.sh' for writing: $0", undef);
 
-                fetchFileFromLink($link, $name);
+        print F "#!" . getGlobal("shell") . "\n";
+        print F "\n";
+        print F getBinDirectoryShellCode();
+        print F "\n";
+        print F setWorkDirectoryShellCode(".");
+        print F "\n";
 
-                $inputs[$ff] = "$inType\0./$name";
-            }
+        print F "\n";
+        print F "$bin/sqStoreCreate \\\n";
+        print F "  -o ./$asm.seqStore.BUILDING \\\n";
+        print F "  -minlength "  . getGlobal("minReadLength")        . " \\\n";
+
+        if (getGlobal("maxInputCoverage") > 0) {
+            print F "  -genomesize " . getGlobal("genomeSize")       . " \\\n";
+            print F "  -coverage   " . getGlobal("maxInputCoverage") . " \\\n";
+            print F "  -bias       " . getGlobal("readSamplingBias") . " \\\n";
         }
 
-        #  Build a ssi file for all the raw sequence inputs.  For simplicity, we just copy in any
-        #  ssi files as is.  This documents what the store was built with, etc.
-
-        open(F, "> ./$asm.seqStore.ssi") or caExit("cant' open './$asm.seqStore.ssi' for writing: $0", undef);
-
         foreach my $iii (@inputs) {
-            if ($iii =~ m/^-(.*)\0(.*)$/) {
+            if ($iii =~ m/^(.*)\0(.*)$/) {
                 my $tech = $1;
                 my $file = $2;
                 my @name = split '/', $2;
@@ -219,95 +270,35 @@ sub createSequenceStore ($$@) {
                 $name = $1   if ($name =~ m/(.*).fast[aq]$/i);
                 $name = $1   if ($name =~ m/(.*).f[aq]$/i);
 
-                print F "########################################\n";
-                print F "#  $tech: $file\n";
-                print F "#\n";
-                print F "name   $name\n";
-                print F "preset $tech\n";
-                print F "$file\n";
-                print F "\n";
-
-            } elsif (-e $iii) {
-                print F "########################################\n";
-                print F "#  $iii\n";
-                print F "#\n";
-                open(I, "< $iii") or caExit("can't open sqStoreCreate input '$iii' for reading: $0", undef);
-                while (<I>) {
-                    print F $_;
-                }
-                close(I);
-                print F "\n";
+                print F "  $tech $name $file \\\n";
 
             } else {
                 caExit("unrecognized sqStoreCreate input file '$iii'", undef);
             }
         }
 
+        print F "&& \\\n";
+        print F "mv ./$asm.seqStore.BUILDING ./$asm.seqStore \\\n";
+        print F "&& \\\n";
+        print F "exit 0\n";
+        print F "\n";
+        print F "exit 1\n";
+
         close(F);
+
+        makeExecutable("./$asm.seqStore.sh");
+        stashFile("./$asm.seqStore.sh");
     }
 
     #  Load the store.
 
-    if (! -e "./$asm.seqStore.BUILDING") {
-        my $cmd;
-        $cmd .= "$bin/sqStoreCreate \\\n";
-        $cmd .= "  -o ./$asm.seqStore.BUILDING \\\n";
-        $cmd .= "  -minlength "  . getGlobal("minReadLength")        . " \\\n";
-        if (getGlobal("readSamplingCoverage") > 0) {
-            $cmd .= "  -genomesize " . getGlobal("genomeSize")           . " \\\n";
-            $cmd .= "  -coverage   " . getGlobal("readSamplingCoverage") . " \\\n";
-            $cmd .= "  -bias       " . getGlobal("readSamplingBias")     . " \\\n";
-        }
-        $cmd .= "  ./$asm.seqStore.ssi \\\n";
-        $cmd .= "> ./$asm.seqStore.BUILDING.err 2>&1";
-
-        if (runCommand(".", $cmd) > 0) {
-            caExit("sqStoreCreate failed", "./$asm.seqStore.BUILDING.err");
-        }
+    if (runCommand(".", "./$asm.seqStore.sh > ./$asm.seqStore.err 2>&1") > 0) {
+        caExit("sqStoreCreate failed; boom!", "./$asm.seqStore.err");
     }
 
-    #  Check for quality issues.
-
-    if (-e "./$asm.seqStore.BUILDING.err") {
-        my $nProblems = 0;
-
-        open(F, "< ./$asm.seqStore.BUILDING.err");
-        while (<F>) {
-            $nProblems++   if (m/Check\syour\sreads/);
-        }
-        close(F);
-
-        if ($nProblems > 0) {
-            if (getGlobal("stopOnReadQuality")) {
-                print STDERR "\n";
-                print STDERR "Potential problems with your input reads were detected.\n";
-                print STDERR "\n";
-                print STDERR "Please review the logging in files:\n";
-                print STDERR "  ", getcwd(), "/$asm.seqStore.BUILDING.err\n";
-                print STDERR "  ", getcwd(), "/$asm.seqStore.BUILDING/errorLog\n";
-                print STDERR "\n";
-                print STDERR "If you wish to proceed, rename the store with the following command and restart canu.\n";
-                print STDERR "\n";
-                print STDERR "  mv ", getcwd(), "/$asm.seqStore.BUILDING \\\n";
-                print STDERR "     ", getcwd(), "/$asm.seqStore.ACCEPTED\n";
-                print STDERR "\n";
-                print STDERR "Option stopOnReadQuality=false skips these checks.\n";
-                exit(1);
-            } else {
-                print STDERR "--\n";
-                print STDERR "-- WARNING:  Potential problems with your input reads were detected.\n";
-                print STDERR "-- WARNING:\n";
-                print STDERR "-- WARNING:  Please review the logging in files:\n";
-                print STDERR "-- WARNING:    ", getcwd(), "/$asm.seqStore.BUILDING.err\n";
-                print STDERR "-- WARNING:    ", getcwd(), "/$asm.seqStore.BUILDING/errorLog\n";
-                print STDERR "-- \n";
-                print STDERR "-- Proceeding with assembly because stopOnReadQuality=false.\n";
-            }
-        }
+    if (! -e "./$asm.seqStore/info.txt") {
+        caExit("sqStoreCreate failed; no info file", "./$asm.seqStore.err");
     }
-
-    rename "./$asm.seqStore.BUILDING",             "./$asm.seqStore";
-    rename "./$asm.seqStore.BUILDING.err",         "./$asm.seqStore.err";
 }
 
 
@@ -325,32 +316,27 @@ sub generateReadLengthHistogram ($$) {
     my @rl;
     my @hi;
 
-    #  Load the read lengths, find min and max lengths.
+    #  Generate a lovely PNG histogram.
 
-    open(F, "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -reads |") or caExit("can't dump meta data from './$asm.seqStore': $!", undef);
-    while (<F>) {
-        next  if (m/readID/);             #  Skip the header.
-        next  if (m/------/);
+    if (! -e "./$asm.seqStore/readlengths-$tag.dat") {
+        my $cmd;
 
-        s/^\s+//;
-        s/\s+$//;
+        $cmd  = "$bin/sqStoreDumpMetaData \\\n";
+        $cmd .= "  -S ./$asm.seqStore \\\n";
+        $cmd .= "  -raw \\\n"                  if ($tag eq "cor");
+        $cmd .= "  -corrected \\\n"            if ($tag eq "obt");
+        $cmd .= "  -corrected -trimmed \\\n"   if ($tag eq "utg");
+        $cmd .= "  -histogram \\\n";
+        $cmd .= "  -lengths \\\n";
+        $cmd .= "> ./$asm.seqStore/readlengths-$tag.dat \\\n";
+        $cmd .= "2> ./$asm.seqStore/readlengths-$tag.err \n";
 
-        my @v = split '\s+', $_;
-        my $l = 0;
+        if (runCommand(".", $cmd) > 0) {
+            caExit("sqStoreDumpMetaData failed", "./$asm.seqStore/readlengths-$tag.err");
+        }
 
-        $l = $v[3]          if (($tag eq "cor") || ($tag eq "hap"));
-        $l = $v[4]          if (($tag eq "obt"));
-        $l = $v[6] - $v[5]  if (($tag eq "utg"));
+        unlink "./$asm.seqStore/readlengths-$tag.err";
 
-        push @rl, $l   if ($l > 0);
-    }
-    close(F);
-
-    @rl = sort { $a <=> $b } @rl;
-
-    #  Generate PNG histograms if there are any reads.
-
-    if ($reads > 0) {
         my $gnuplot = getGlobal("gnuplot");
         my $format  = getGlobal("gnuplotImageFormat");
 
@@ -369,12 +355,6 @@ sub generateReadLengthHistogram ($$) {
             print F "plot [] './$asm.seqStore/readlengths-$tag.dat' using (bin(\$1,binwidth)):(1.0) smooth freq with boxes title ''\n";
             close(F);
 
-            open(F, "> ./$asm.seqStore/readlengths-$tag.dat") or caExit("can't open './$asm.seqStore/readlengths-$tag.dat' for writing: $!", undef);
-            foreach my $rl (@rl) {
-                print F "$rl\n";
-            }
-            close(F);
-
             if (runCommandSilently(".", "$gnuplot < /dev/null ./$asm.seqStore/readlengths-$tag.gp > /dev/null 2>&1", 0)) {
                 print STDERR "--\n";
                 print STDERR "-- WARNING: gnuplot failed.\n";
@@ -384,72 +364,43 @@ sub generateReadLengthHistogram ($$) {
         }
     }
 
-    #  Generate the ASCII histogram.
+    #  Generate a lovely ASCII histogram.
+
+    if (! -e "./$asm.seqStore/readlengths-$tag.txt") {
+        my $cmd;
+
+        $cmd  = "$bin/sqStoreDumpMetaData \\\n";
+        $cmd .= "  -S ./$asm.seqStore \\\n";
+        $cmd .= "  -raw \\\n"                  if ($tag eq "cor");
+        $cmd .= "  -corrected \\\n"            if ($tag eq "obt");
+        $cmd .= "  -corrected -trimmed \\\n"   if ($tag eq "utg");
+        $cmd .= "  -histogram \\\n";
+        $cmd .= "> ./$asm.seqStore/readlengths-$tag.txt \\\n";
+        $cmd .= "2> ./$asm.seqStore/readlengths-$tag.err \n";
+
+        if (runCommand(".", $cmd) > 0) {
+            caExit("sqStoreDumpMetaData failed", "./$asm.seqStore/readlengths-$tag.err");
+        }
+
+        unlink "./$asm.seqStore/readlengths-$tag.err";
+    }
+
+    #  Read the ASCII histogram, append to report.
 
     $hist  = "--\n";
     $hist .= "-- In sequence store './$asm.seqStore':\n";
     $hist .= "--   Found $reads reads.\n";
     $hist .= "--   Found $bases bases ($coverage times coverage).\n";
 
-    if ($reads > 0) {
-        my $minLen     = $rl[ 0];
-        my $maxLen     = $rl[-1];
-        my $scale      = 0;
-        my $bucketSize = 0;
+    if (-e "./$asm.seqStore/readlengths-$tag.txt") {
+        #$hist .= "--\n";
+        #$hist .= "--   Read length histogram (one '*' equals " . int(100 * $scale) / 100 . " reads):\n";
 
-        #  Buckets of size 1000 are easy to interpret, but sometimes not ideal.
-
-        $bucketSize = 10000;
-        $bucketSize = 5000    if ($maxLen - $minLen < 1000000);
-        $bucketSize = 1000    if ($maxLen - $minLen < 100000);
-        $bucketSize = 100     if ($maxLen - $minLen < 10000);
-
-        #  Generate the histogram (int truncates)
-
-        my $mBgn = int($minLen / $bucketSize);
-        my $mEnd = int($maxLen / $bucketSize);
-
-        foreach my $rl (@rl) {
-            my $b = int($rl / $bucketSize);
-
-            $hi[$b]++;
+        open(F, "< ./$asm.seqStore/readlengths-$tag.txt") or caExit("can't open './$asm.seqStore/readlengths-$tag.txt' for reading: $!", undef);
+        while (<F>) {
+            $hist .= "--    $_";
         }
-
-        for (my $ii=$mBgn; $ii<=$mEnd; $ii++) {                           #  Scale the *'s so that the longest has 70 of 'em
-            $scale = $hi[$ii] / 70   if ($scale < $hi[$ii] / 70);
-        }
-
-        #  Draw the histogram.
-
-        $hist .= "--\n";
-        $hist .= "--   Read length histogram (one '*' equals " . int(100 * $scale) / 100 . " reads):\n";
-
-        for (my $ii=$mBgn; $ii<=$mEnd; $ii++) {
-            my $s = $ii * $bucketSize;
-            my $e = $ii * $bucketSize + $bucketSize - 1;
-
-            $hi[$ii] += 0;  #  Otherwise, cells with no count print as null.
-
-            $hist .= sprintf("--   %6d %6d %6d %s\n", $s, $e, $hi[$ii], "*" x int($hi[$ii] / $scale));
-        }
-    }
-
-    #  Abort if the read coverage is too low.
-
-    my $minCov = getGlobal("stopOnLowCoverage");
-
-    if ($coverage < $minCov) {
-        print STDERR "--\n";
-        print STDERR "-- ERROR:  Read coverage ($coverage) is too low to be useful.\n";
-        print STDERR "-- ERROR:\n";
-        print STDERR "-- ERROR:  This could be caused by an incorrect genomeSize or poor quality reads that could not\n";
-        print STDERR "-- ERROR:  be sufficiently corrected.\n";
-        print STDERR "-- ERROR:\n";
-        print STDERR "-- ERROR:  You can force Canu to continue by decreasing parameter stopOnLowCoverage=$minCov,\n";
-        print STDERR "-- ERROR:  however, the quality of corrected reads and/or contiguity of contigs will be poor.\n";
-        print STDERR "-- \n";
-
-        caExit("", undef);
+        close(F);
     }
 
     #  Return the ASCII histogram.
@@ -483,26 +434,28 @@ sub checkSequenceStore ($$@) {
     #goto allDone    if (getNumberOfReadsInStore($asm, $tag) > 0);
 
     #  Create the store.
-    #
-    #  If all goes well, we get asm.seqStore.
-    #
-    #  If not, we could end up with asm.seqStore.BUILDING and ask the user to examine it and rename
-    #  it to asm.seqStore.ACCEPTED and restart.  On the restart, sqStoreCreateStore() detects the
-    #  'ACCPETED' store and renames to asm.seqStore.
 
+    my $storeCreated = 0;
     my $histAndStash = 0;
 
     if (! -e "./$asm.seqStore") {
         createSequenceStore($base, $asm, @inputs);
 
+        $storeCreated = 1;
         $histAndStash = 1;
     }
 
     #  Dump the list of libraries.  Various parts use this for various stuff.
 
+    if (! -e "./$asm.seqStore/info.txt") {
+        if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -stats > ./$asm.seqStore/info.txt 2> /dev/null", 1)) {
+            caExit("failed to generate $asm.seqStore/info.txt", undef);
+        }
+    }
+
     if (! -e "./$asm.seqStore/libraries.txt") {
         if (runCommandSilently(".", "$bin/sqStoreDumpMetaData -S ./$asm.seqStore -libs > ./$asm.seqStore/libraries.txt 2> /dev/null", 1)) {
-            caExit("failed to generate list of libraries in store", undef);
+            caExit("failed to generate $asm.seqStore/libraries.txt", undef);
         }
     }
 
@@ -524,27 +477,12 @@ sub checkSequenceStore ($$@) {
         caExit("unable to assemble uncorrected reads", undef);
     }
 
-    #  Promote corrected reads to trimmed reads, if needed.
+    #  Set or unset the homopolymer compression flag.
 
-    if (($tag eq "utg") &&    #  Assembling.
-        ($nOBT > 0) &&        #  Corrected reads exist.
-        ($nAsm == 0)) {       #  But no trimmed reads exist.
-
-        print STDERR "--\n";
-        print STDERR "-- WARNING:  No trimmed reads found for assembly, but untrimmed reads exist.\n";
-        print STDERR "-- WARNING:  Upgrading untrimmed reads to trimmed reads for assembly.\n";
-
-        if (runCommandSilently(".", "$bin/loadTrimmedReads -S ./$asm.seqStore > ./$asm.seqStore.upgrade.err 2>&1", 1)) {
-            caExit("initializing clear ranges failed", "./$asm.seqStore.upgrade.err");
-        }
-
-        unlink "./$asm.seqStore.upgrade.err";
-
-        $nCor = getNumberOfReadsInStore($asm, "cor");   #  Number of corrected reads ready for OBT.
-        $nOBT = getNumberOfReadsInStore($asm, "obt");   #  Number of corrected reads ready for OBT.
-        $nAsm = getNumberOfReadsInStore($asm, "utg");   #  Number of trimmed reads ready for assembly.
-
-        $histAndStash = 1;
+    if (defined(getGlobal("homoPolyCompress"))) {
+        touch("./$asm.seqStore/homopolymerCompression");
+    } else {
+        unlink("./$asm.seqStore/homopolymerCompression");
     }
 
     #  Make a histogram and stash the (updated) store.
@@ -560,7 +498,7 @@ sub checkSequenceStore ($$@) {
         (($tag eq "obt") && ($nOBT == 0)) ||
         (($tag eq "utg") && ($nAsm == 0))) {
 
-        print STDERR "--\n";
+        print STDERR "--\n";              #  Nothing clever here, just keeping text aligned.
         print STDERR "-- WARNING:\n";
         print STDERR "-- WARNING:  No raw ",       "reads detected.  Cannot proceed; empty outputs generated.\n"  if ($tag eq "cor");
         print STDERR "-- WARNING:  No corrected ", "reads detected.  Cannot proceed; empty outputs generated.\n"  if ($tag eq "obt");
@@ -573,6 +511,31 @@ sub checkSequenceStore ($$@) {
         return(0);
     }
 
+    #  Stop if there isn't enough coverage.
+
+    my $minInputCov = getGlobal("minInputCoverage");
+    my $minStoreCov = getGlobal("stopOnLowCoverage");
+
+    my $coverage = int(100 * getNumberOfBasesInStore($asm, $tag) / getGlobal("genomeSize")) / 100;
+
+    if ((($coverage < $minInputCov) && ($storeCreated)) ||
+        (($coverage < $minStoreCov))) {
+        print STDERR "--\n";
+        print STDERR "-- ERROR:  Read coverage ($coverage) lower than allowed.\n";
+        print STDERR "-- ERROR:    minInputCoverage  = $minInputCov\n";
+        print STDERR "-- ERROR:    stopOnLowCoverage = $minStoreCov\n";
+        print STDERR "-- ERROR:\n";
+        print STDERR "-- ERROR:  This could be caused by an incorrect genomeSize or poor quality\n";
+        print STDERR "-- ERROR:  reads that cound not be sufficiently corrected.\n";
+        print STDERR "-- ERROR:\n";
+        print STDERR "-- ERROR:  You can force Canu to continue by decreasing parameters\n";
+        print STDERR "-- ERROR:  minInputCoverage and/or stopOnLowCoverage.  Be warned that the\n";
+        print STDERR "-- ERROR:  quality of corrected reads and/or contiguity of contigs will be poor.\n";
+        print STDERR "-- \n";
+
+        caExit("", undef);
+    }
+
   finishStage:
     generateReport($asm);
 
